<?php
/**
 * Initialises the app
 *
 * @package Fast Plugin
 *
 * @copyright 2019 Oxygenna.com
 * @license http://themeforest.net/licenses/standard
 * @version 1.15.4
 * @author Oxygenna
 *
 * Ever wanted a clean slate on the WordPress front end?
 * This class strips the entire page. All styles and scripts are removed.
 *
 * Usage:
 * 1. Call `FastCleanSlate::init();`
 * 2. Enqueue scripts and styles via wp_enqueue_scripts_clean
 * 3. Output custom content with wp_clean_slate_content
 *
 * The following hooks are available:
 *     wp_enqueue_scripts_clean  Enqueue your styles here
 *     wp_clean_slate_content    Output any content you want following the opening body tag
 *     wp_clean_slate_options    Override default options (filter)
 *     wp_clean_slate_head       Outputs before wp_head
 *     wp_clean_slate_footer     Outputs after wp_footer
 */

/**
 * Wipes all WP actions and filters
 */
class FastCleanSlate {

    /**
     * I am a singleton
     *
     * @var object
     */
    private static $instance;

    /**
     * Filterable options for enabling / disabling various outputs
     *
     * @var array
     */
    private $options;

    /**
     * Instantiate class with filterable defaults
     */
    public function __construct()
    {
        $this->options = apply_filters('wp_clean_slate_options', array(
            'add_title' => true,
            'add_charset' => true,
            'add_viewport' => true,
            'add_body_classes' => false,
            'inject_template' => true, // Set false if you are filtering template_include yourself
            'remove_jquery_migrate' => false, // We have full control, so we can probaly be sure we're using up to date jQuery APIs.
            'media_templates' => true,
            'reset_footer' => true,
            'show_admin_bar' => false
        ));

        if ($this->options['remove_jquery_migrate']) {
            add_action('wp_default_scripts', array($this, 'remove_jquery_migrate'));
        }

        add_action('template_redirect', array($this, 'hooks'), 99999);
    }

    /**
     * Attach required actions and filters
     * Strip WP actions
     *
     * @return void
     */
    public function hooks()
    {

        if ($this->options['add_title']) {
            add_action('wp_clean_slate_head', array($this, 'add_title'));
        }

        if ($this->options['add_charset']) {
            add_action('wp_clean_slate_head', array($this, 'add_charset'));
        }

        if ($this->options['add_viewport']) {
            add_action('wp_clean_slate_head', array($this, 'add_viewport'));
        }

        // Override Theme.
        add_filter('template_include', array($this, 'inject_template'), 99999);

        // Remove ALL actions to strip 3rd party plugins and unwanted WP functions.
        remove_all_actions('wp_head');
        remove_all_actions('wp_print_styles');
        remove_all_actions('wp_print_head_scripts');

        // Add back WP native actions that we need.
        add_action('wp_head', 'wp_enqueue_scripts', 1);
        add_action('wp_head', 'wp_print_styles', 8);
        add_action('wp_head', 'wp_print_head_scripts', 9);

        // No Toolbar.
        if (false === $this->options['show_admin_bar']) {
            add_filter('show_admin_bar', '__return_false');
        } else {
            if (!class_exists('WP_Admin_Bar')) {
                _wp_admin_bar_init();
            }
            add_action('wp_enqueue_scripts_clean', array($this, 'admin_bar_enqueue'));
        }

        // Strip all scripts and styles.
        add_action('wp_head', array($this, 'strip_enqueues'), -1);
    }

    /**
     * Output our header, content, and footer.
     * Returning false will also block WordPress from including a file
     *
     * @return boolean  always false
     */
    public function inject_template()
    {
        $this->render_header();
        do_action('wp_clean_slate_content');
        $this->render_footer();
        return false;
    }

    /**
     * Output opening HTML with wp_head and optional elements
     *
     * @return void
     */
    public function render_header()
    {
        include FAST_PLUGIN_DIR . 'templates/clean/header.php';
    }

    /**
     * Output wp_footer and closing HTML
     *
     * @return void
     */
    public function render_footer()
    {
        if ($this->options['reset_footer']) {
            $this->reset_footer();
        }
        include FAST_PLUGIN_DIR . 'templates/clean/footer.php';
    }

    /**
     * Output wp_title
     */
    public function add_title()
    {
        echo '<title>';
        echo get_the_title();
        echo '</title>';
    }

    /**
     * Output charset meta tag
     */
    public function add_charset()
    {
        echo '<meta charset="';
        bloginfo('charset');
        echo '">';
    }

    /**
     * Output viewport meta tag
     */
    public function add_viewport()
    {
        echo '<meta name="viewport" content="width=device-width, initial-scale=1.0">';
    }

    /**
     * Remove jQuery Migrate as a dependency from jQuery
     *
     * @param object $scripts WordPress scripts.
     */
    public function remove_jquery_migrate(&$scripts)
    {
        $jquery = $scripts->registered['jquery'];
        $version = $jquery->ver;

        $scripts->remove('jquery');
        $scripts->remove('jquery-migrate');
        $scripts->add('jquery', false, array('jquery-core'), $version);
    }

    /**
     * Remove all enqueue actions as early as possible
     */
    public function strip_enqueues()
    {
        remove_all_actions('wp_enqueue_scripts');
        add_action('wp_enqueue_scripts', array($this, 'reset_enqueues'), 999999);
    }

    /**
     * Reset the style and script registries in case anything is still registered
     */
    public function reset_enqueues()
    {
        global $wp_styles;
        global $wp_scripts;

        // @codingStandardsIgnoreStart
        $wp_styles = new WP_Styles();
        $wp_scripts = new WP_Scripts();
        // @codingStandardsIgnoreEnd

        do_action('wp_enqueue_scripts_clean');
    }

    /**
     * Reset wp_footer to the minimum
     */
    public function reset_footer()
    {
        remove_all_actions('wp_footer');

        add_action('wp_footer', 'wp_print_footer_scripts', 20);
        add_action('wp_footer', 'wp_admin_bar_render', 1000);
    }

    /**
     * Re-enqueue the Admin bar scripts and styles
     *
     * @return void
     */
    public function admin_bar_enqueue()
    {
        wp_enqueue_script('admin-bar');
        wp_enqueue_style('admin-bar');
    }

    /**
     * Instance accessor. If instance doesn't exist, we'll initialize the class.
     *
     * @return object WP_CleanFastCleanSlate_Slate::$instance
     */
    public static function instance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new FastCleanSlate;
        }
        return self::$instance;
    }

    /**
     * Alias for ::instance
     * For semantics. init can be called when the intention is the first initialization
     *
     * @return object FastCleanSlate::$instance
     */
    public static function init()
    {
        return self::instance();
    }
}
